package xyz.lychee.lagfixer.libs.adventure.text;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Objects;
import java.util.function.Predicate;
import xyz.lychee.lagfixer.libs.annotations.Contract;
import xyz.lychee.lagfixer.libs.annotations.NotNull;
import xyz.lychee.lagfixer.libs.annotations.Nullable;

@FunctionalInterface
public interface ComponentLike {
   static @NotNull List<Component> asComponents(final @NotNull List<? extends ComponentLike> likes) {
      return asComponents(likes, (Predicate)null);
   }

   @NotNull
   static List<Component> asComponents(@NotNull final List<? extends ComponentLike> likes, @Nullable final Predicate<? super Component> filter) {
      Objects.requireNonNull(likes, "likes");
      int size = likes.size();
      if (size == 0) {
         return Collections.emptyList();
      } else {
         ArrayList<Component> components = null;

         for(int i = 0; i < size; ++i) {
            ComponentLike like = (ComponentLike)likes.get(i);
            if (like == null) {
               throw new NullPointerException("likes[" + i + "]");
            }

            Component component = like.asComponent();
            if (filter == null || filter.test(component)) {
               if (components == null) {
                  components = new ArrayList(size);
               }

               components.add(component);
            }
         }

         if (components == null) {
            return Collections.emptyList();
         } else {
            components.trimToSize();
            return Collections.unmodifiableList(components);
         }
      }
   }

   static @Nullable Component unbox(final @Nullable ComponentLike like) {
      return like != null ? like.asComponent() : null;
   }

   @Contract(
      pure = true
   )
   @NotNull Component asComponent();
}
